/*------------------------------------------------------------------------
    File        : customer_abl_gui.p
    Purpose     : 

    Syntax      :

    Description : 

    Author(s)   : pjudge
    Created     : Wed Jun 08 08:47:39 EDT 2011
    Notes       :
  ----------------------------------------------------------------------*/

/* ***************************  Definitions  ************************** */

routine-level on error undo, throw.

/* ********************  Preprocessor Definitions  ******************** */


/* ***************************  Main Block  *************************** */
/* A. UI Definitions */
define button btnFirst label '|<' font 8.
define button btnPrev label '<' font 8.
define button btnNext label '>' font 8.
define button btnLast label '>|' font 8.

define button btnAdd label 'Add' font 8.
define button btnSave label 'Save' font 8.
define button btnDelete label 'Delete' font 8.

define temp-table ttCustomer no-undo before-table btCustomer
    field CustNum as integer
    field name as character
    field CreditLimit as decimal
    field Balance as decimal
    field FullAddress as character
    index idx1 as primary unique CustNum.

define frame fCustomer
    ttCustomer.CustNum font 8
    ttCustomer.CreditLimit font 8 
    ttCustomer.Balance font 8
    ttCustomer.FullAddress format 'x(50)' font 8
    with row 5 width 200 1 col.
    
define frame fNavPanel
    btnFirst
    btnPrev
    btnNext
    btnLast
    with row 1.

define frame fUpdatePanel
    btnAdd
    btnSave
    btnDelete
    with row 1 col 50 width 60.
    
/* B. UI trigger definitions */
on choose of btnFirst in frame fNavPanel
do:
    run GetRecord('first').
end.

on choose of btnPrev in frame fNavPanel
do:
    run GetRecord('prev').
end.

on choose of btnNext in frame fNavPanel
do:
    run GetRecord('next').
end.

on choose of btnLast in frame fNavPanel
do:
    run GetRecord('last').
end.
/* C. trigger contents */
define variable mhEventHandler as handle no-undo.

procedure GetRecord:
    define input parameter pcPosition as character no-undo.
    
    case pcPosition:
        when 'first' then run GetFirst in mhEventHandler ('customer').
        when 'prev' then run GetPrev in mhEventHandler('customer').
        when 'next' then run GetNext in mhEventHandler ('customer').
        when 'last' then run GetLast in mhEventHandler ('customer').
    end case.
end procedure.

/* D. Remainder */
procedure Initialise:
    current-window:width-c = 200.
    enable all with frame fNavPanel.
end procedure.

procedure Destroy:
    
end procedure.

procedure ShowUI:
    view frame fNavPanel.
    view frame fUpdatePanel.
    view frame fCustomer.
    
    current-window:title = this-procedure:file-name.
end procedure.

procedure HideUI:
    hide frame hNavPanel.
    hide frame fUpdatePanel.
    hide frame fCustomer.
    
    current-window:hidden = true.
end procedure.

procedure SetEventHandler:
    define input  parameter phEventHandler as handle no-undo.
    
    mhEventHandler = phEventHandler.
end procedure.

procedure SetNavPanelState:
    define input  parameter plEnabled as logical extent 4 no-undo.
    
    assign 
        btnFirst:sensitive in frame fNavPanel = plEnabled[1]
        btnPrev:sensitive in frame fNavPanel = plEnabled[2]
        btnNext:sensitive in frame fNavPanel = plEnabled[3]
        btnLast:sensitive in frame fNavPanel = plEnabled[4]
        .
end procedure.

on close of this-procedure
do:
    run EndWaitFor in mhEventHandler.
    return no-apply.
end.

procedure BeginWaitFor:
    define input parameter pcShowMode as character no-undo.
    
    /* ABL GUI doesn't give us the option easily to do modal */
    wait-for close of this-procedure.
end procedure.

procedure EndWaitFor:
    apply 'close' to this-procedure.
end procedure.

procedure ShowRecord:
    define variable cField as character extent 4 no-undo.
    define variable cValue as character extent no-undo.
    
    assign cField[1] = 'CustNum'
           cField[2] = 'FullAddress'
           cField[3] = 'CreditLimit'
           cField[4] = 'Balance'.
    
    run GetDataValue in mhEventHandler (
        'customer',
        cField,
        output cValue).
    
    displ int(cValue[1])     @ ttCustomer.CustNum
          decimal(cValue[3]) @ ttCustomer.CreditLimit
          decimal(cValue[4]) @ ttCustomer.Balance
          cValue[2]          @ ttCustomer.FullAddress
            with frame fCustomer.    
end procedure.


procedure GetScreenValue:
    define input  parameter pcFieldName as character extent no-undo.
    define output parameter pcFieldValue as character extent no-undo.
    
    define variable iLoop as integer no-undo.
    define variable iMax as integer no-undo.
    
    iMax = extent(pcFieldName).
    extent(pcFieldValue) = iMax.
    do iLoop = 1 to iMax with frame fCustomer:
        case pcFieldName[iLoop]:
            when 'CustNum' then pcFieldValue[iLoop] = ttCustomer.CustNum:screen-value.
            when 'FullAddress' then pcFieldValue[iLoop] = ttCustomer.FullAddress:screen-value.
            when 'CreditLimit' then pcFieldValue[iLoop] = ttCustomer.CreditLimit:screen-value.
            when 'Balance' then pcFieldValue[iLoop] = ttCustomer.Balance:screen-value.
        end case.            
    end.
end procedure.

    
    