
/*------------------------------------------------------------------------
    File        : customer_clientdata.p
    Purpose     : 

    Syntax      :

    Description : 

    Author(s)   : pjudge
    Created     : Wed Jun 08 08:40:38 EDT 2011
    Notes       :
  ----------------------------------------------------------------------*/

/* ***************************  Definitions  ************************** */

routine-level on error undo, throw.

/* ********************  Preprocessor Definitions  ******************** */


/* ***************************  Main Block  *************************** */
define temp-table ttCustomer no-undo before-table btCustomer
    field CustNum       as integer
    field name          as character
    field CreditLimit   as decimal
    field Balance       as decimal
    field FullAddress as character
    index idx1 as primary unique CustNum.

define temp-table ttTransportCustomer no-undo like ttCustomer.

define query qryCust for ttCustomer scrolling.
define variable hAppServer as handle no-undo.    

/* A. INIT/DESTROY */
procedure Initialise:
    create server hAppServer.
    hAppServer:connect('-AppService asPugChallenge11 -sessionModel session-free').
        
    run FetchData.
end procedure.

procedure Destroy:
    hAppServer:disconnect().
    delete object hAppServer.
end procedure.

/* B. get and save DATA */
procedure FetchData:
    run data/GetCustomers.p on hAppServer (
        input 'where custnum le 100',
        output table ttCustomer).
    
    run OpenQuery.
end procedure.

procedure SaveData:
    /* only send changes across the wire */
    buffer ttTransportCustomer:get-changes(buffer ttCustomer:handle).
    
    run data/SaveCustomers.p on hAppServer (input table ttTransportCustomer).
    
    /* should really do error handling here! but, it compiles, so ... */
    
    run OpenQuery.
end procedure.

/* C. BASIC DATA NAVIGATION */
procedure OpenQuery:
    open query qryCust preselect each ttCustomer.
end procedure.

procedure CloseQuery:
    if query qryCust:is-open then
        close query qryCust.
end procedure.

procedure GetFirst:
    query qryCust:get-first().
end procedure.

procedure GetPrev:
    query qryCust:get-prev().
end procedure.

procedure GetNext:
    query qryCust:get-next().
end procedure.

procedure GetLast:
    query qryCust:get-last().
end procedure.

/* D. DATA INPUT/OUTPUT from UI */
procedure GetDataValue:
    define input  parameter pcFieldName as character extent no-undo.
    define output parameter pcValue as character extent no-undo.
    define variable iLoop   as integer no-undo.
    define variable iMax    as integer no-undo.
    define variable hBuffer as handle  no-undo.
    
    iMax = extent (pcFieldName).
    extent(pcValue) = iMax.
    do iLoop = 1 to iMax:
        pcValue[iLoop] = buffer ttCustomer:buffer-field(pcFieldName[iLoop]):buffer-value.
    end.
end procedure.

procedure SetValue:
    define input parameter pcFieldName as character extent no-undo.
    define input parameter pcValue as character extent no-undo.
    
    define variable iLoop   as integer no-undo.
    define variable iMax    as integer no-undo.
    define variable hBuffer as handle  no-undo.
    
    iMax = extent (pcFieldName).
    extent(pcValue) = iMax.
    do iLoop = 1 to iMax:
        buffer ttCustomer:buffer-field(pcFieldName[iLoop]):buffer-value = pcValue[iLoop].
    end.
end procedure.

procedure GetQueryHandle:
    define output parameter phQuery as handle no-undo.
    
    phQuery =  query qryCust:handle.
end procedure.

/* E. want extra enhancements - filtering, sorting, navigating around result set etc */
procedure RepositionQuery:
    define input parameter pcFieldName as character no-undo.
    define input parameter pcValue as character no-undo.

    define buffer lbRepos for ttCustomer.
    
    case pcFieldName:
        when 'CustNum' then 
            find lbRepos where lbRepos.CustNum eq int(pcValue) no-error.
    end.
    
    if available lbRepos then
    do:
        query qryCust:reposition-to-rowid(rowid(lbRepos)).
        query qryCust:get-next().
    end.
end procedure.

procedure FilterQuery:
    define input parameter pcFilter as character.
    
    define variable hQuery as handle no-undo.
    
    hQuery = query qryCust:handle.
    
    hQuery:query-prepare(substitute(' preselect each ttCustomer &1', pcFilter)).
    
    run OpenQuery.
end procedure.

procedure SortQuery:
    define input parameter pcSort as character.
    define variable hQuery as handle no-undo.
    
    hQuery = query qryCust:handle.
    
    hQuery:query-prepare(substitute(' preselect each ttCustomer by &1', pcSort)).
    
    run OpenQuery.
end procedure.

procedure GetCurrentPosition:
    define output parameter pcCurrentPosition as character no-undo.

    /* IsFirst */
    if not query qryCust:is-open or
        query qryCust:current-result-row eq 0 then
        pcCurrentPosition = 'None'.
    else       
        /*IsLast or IsFirstAndLast */
        if query qryCust:current-result-row eq query qryCust:num-results then
        do:
            /* IsFirstAndLast */
            if query qryCust:current-result-row eq 1 then
                pcCurrentPosition = 'IsFirstAndLast'. 
            else
                pcCurrentPosition = 'IsLast'.
        end.
        else
            /* IsFirst */
            if query qryCust:current-result-row eq 1 then
                pcCurrentPosition = 'IsFirst'. 
            else
                pcCurrentPosition = 'NotFirstOrLast'.               
end procedure.