/*------------------------------------------------------------------------
    File        : customer_eventhandler.p
    Purpose     : Presentation Layer Event Handler for Customer functionality 
    Author(s)   : pjudge
    Created     : Tue May 10 16:13:56 EDT 2011
    Notes       :
  ----------------------------------------------------------------------*/
routine-level on error undo, throw.

/* ***************************  Definitions  ************************** */
define temp-table ttComponent no-undo
    field Type as character /* CLIENTDATA, UI */
    field Name as character /* logical name */
    field FileName as character /* .P, .W  name */
    field Handle as handle
    index idx1 as primary unique Type Name
    . 

define variable mcUIName as character no-undo.
define variable mlShowOnInit as logical no-undo init true.

/** -- functions -- **/
function GetCurrentUI returns handle ():
    find ttComponent where
         ttComponent.Type eq 'UI' and
         ttComponent.Name eq mcUIName 
         no-lock no-error.
    if available ttComponent then
        return ttComponent.Handle.
end function.

/** -- main block --  **/
/* WEBSPEED */
run MainBlock.

procedure MainBlock:
    mlShowOnInit = not (session:client-type eq 'WEBSPEED').
    
    if session:client-type eq 'WEBSPEED' then
    do:
                  
    create ttComponent.
    assign ttComponent.Type = 'UI'
           ttComponent.Name = 'WEBSPEED'
           ttComponent.FileName = 'customer_web_ui.p'
           .
    
        run SetUIName('WEBSPEED').
        run Initialise.
        run ProcessUiEvent.

        /* always push the ui, since there's no event loop */
        run ShowUI.
    end.
end procedure.

procedure ProcessUiEvent:
/*------------------------------------------------------------------------------
  Purpose:     process the web request.
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    define variable cQueryString as character no-undo.
    
    define variable cFieldName as character extent 3 no-undo.
    define variable cFieldValue as character extent 3 no-undo.
    
    assign cFieldName[1] = 'CurrentCustNum'
           cFieldName[2] = 'NavPanel'
           cFieldName[3] = 'UpdatePanel'.
    
    run GetScreenValue in GetCurrentUI() (input cFieldName, output cFieldValue).
    
    case cFieldValue[2]:
        when 'next' then
        do:
            if cFieldValue[1] ne '' then 
                run RepositionQuery('customer', 'CustNum', cFieldValue[1]).
             run GetNext('customer').
        end.
        when 'last' then
        do:
            if cFieldValue[1] ne '' then 
                run RepositionQuery('customer', 'CustNum', cFieldValue[1]).
            run GetLast('customer').
        end.
        when 'prev' then
            run GetPrev('customer').
        otherwise
            run GetFirst('customer').
    end case.
end procedure.

/** -- procedures/ functions -- **/
procedure RegisterComponents:
    /* registered components */
    create ttComponent.
    assign ttComponent.Type = 'CLIENTDATA'
           ttComponent.Name = 'customer'
           ttComponent.FileName = 'customer_clientdata.p'
           .
    
    create ttComponent.
    assign ttComponent.Type = 'UI'
           ttComponent.Name = 'ABLGUI'
           ttComponent.FileName = 'customer_abl_gui.p'
           .
    
    /* A. new UI type */    
    create ttComponent.
    assign ttComponent.Type = 'UI'
           ttComponent.Name = 'DOTNET'
           ttComponent.FileName = 'customer_netgui_wrapper.p'
           .
end procedure.

procedure UpdateButtonState:
    define input parameter phClientData as handle no-undo.
    
    /* [first][prev][next][last] */
    define variable lEnabled as logical extent 4 no-undo.
    define variable cRowPosition as character no-undo.
    
    run GetCurrentPosition in phClientData (output cRowPosition).

    /* Decide what the logical state of the buttons are based on CLIENTDATA info */        
    case cRowPosition:
        when 'IsFirstAndLast ' then
            assign lEnabled[1] = false
                   lEnabled[2] = false
                   lEnabled[3] = false
                   lEnabled[4] = false.
        when 'IsLast' then
            assign lEnabled[1] = true
                   lEnabled[2] = true
                   lEnabled[3] = false
                   lEnabled[4] = false.
       when 'IsFirst' then
        assign lEnabled[1] = false
               lEnabled[2] = false
               lEnabled[3] = true
               lEnabled[4] = true.
        when 'NotFirstOrLast' then               
        assign lEnabled[1] = true
               lEnabled[2] = true
               lEnabled[3] = true
               lEnabled[4] = true.
        otherwise
            assign lEnabled[1] = false
                   lEnabled[2] = false
                   lEnabled[3] = false
                   lEnabled[4] = false.
    end case.
        
    /* tell the UI what to do */
    run SetNavPanelState in GetCurrentUI() (lEnabled).
end procedure.

procedure SetUIName:
    define input  parameter pcUIName as character no-undo.
    
    mcUIName = pcUIName.
end procedure.

procedure BeginWaitFor:
    define input parameter pcShowMode as character no-undo.
    
    run BeginWaitFor in GetCurrentUI() (pcShowMode).
end procedure.

procedure EndWaitFor:
    run EndWaitFor in GetCurrentUI().
    run Destroy.
end procedure.

procedure ShowUI:
    run ShowUI in GetCurrentUI().
end procedure.

procedure Destroy:
    find ttComponent where 
         ttComponent.Type eq 'UI' and
         ttComponent.Name eq mcUIName
         no-error.
    if available ttComponent and valid-handle(ttComponent.Handle) then
    do:
        run Destroy in GetCurrentUI().
        ttComponent.Handle = ?.
    end.
    
    for each ttComponent where ttComponent.Type eq 'CLIENTDATA' and valid-handle(ttComponent.Handle):
        run Destroy in ttComponent.Handle.
        ttComponent.Handle = ?.
    end.
end procedure.

procedure Initialise:
    run RegisterComponents.
    
    /* run and initialise all the components */
    run InitialiseComponents.
    
    /* We need a record to start at */
    run GetFirst('customer').
    
    /* show the UI */
    if mlShowOnInit then
        run ShowUI.
end procedure.

procedure InitialiseComponents:
    for each ttComponent where ttComponent.Type eq 'CLIENTDATA':
        run value(ttComponent.FileName) persistent set ttComponent.Handle.
        run Initialise in ttComponent.Handle.
    end.

    find ttComponent where
         ttComponent.Type eq 'UI' and
         ttComponent.Name eq mcUIName 
         no-lock no-error.
    if available ttComponent then
    do:
        run value(ttComponent.FileName) persistent set ttComponent.Handle.
        run SetEventHandler in ttComponent.Handle (target-procedure).
        run Initialise in ttComponent.Handle.
    end.
end procedure.

procedure GetFirst:
    define input  parameter pcClientDataName as character no-undo.
    
    find ttComponent where 
         ttComponent.Type eq 'CLIENTDATA' and
         ttComponent.Name eq pcClientDataName 
         no-lock no-error.
    if available ttComponent then
    do:
        /* reposition the model to the correct record */
        run GetFirst in ttComponent.Handle.
        
        /* tell the view to get the data it needs */
        run ShowRecord in GetCurrentUI().
        
        /* apply UI logic to the navigation panel buttons */
        run UpdateButtonState(ttComponent.Handle).
    end.
end procedure.

procedure GetPrev:
    define input  parameter pcClientDataName as character no-undo.
    
    find ttComponent where 
         ttComponent.Type eq 'CLIENTDATA' and
         ttComponent.Name eq pcClientDataName 
         no-lock no-error.
    if available ttComponent then
    do:    
        run GetPrev in ttComponent.Handle.
        run ShowRecord in GetCurrentUI().
        run UpdateButtonState(ttComponent.Handle).
    end.                   
end procedure.

procedure GetNext:
    define input  parameter pcClientDataName as character no-undo.
    
    find ttComponent where 
         ttComponent.Type eq 'CLIENTDATA' and
         ttComponent.Name eq pcClientDataName 
         no-lock no-error.
    if available ttComponent then
    do:    
        run GetNext in ttComponent.Handle.
        run ShowRecord in GetCurrentUI().
        run UpdateButtonState(ttComponent.Handle).
    end.                   
end procedure.

procedure GetLast:
    define input  parameter pcClientDataName as character no-undo.
    
    find ttComponent where 
         ttComponent.Type eq 'CLIENTDATA' and
         ttComponent.Name eq pcClientDataName 
         no-lock no-error.
    if available ttComponent then
    do:
        run GetLast in ttComponent.Handle.
        run ShowRecord in GetCurrentUI(). 
        run UpdateButtonState(ttComponent.Handle).
    end.
end procedure.

procedure GetDataValue:
    define input  parameter pcClientDataName as character no-undo.
    define input  parameter pcFieldName as character extent no-undo.
    define output parameter pcValue as character extent no-undo.
    
    find ttComponent where 
         ttComponent.Type eq 'CLIENTDATA' and
         ttComponent.Name eq pcClientDataName 
         no-lock no-error.
    if available ttComponent then
        run GetDataValue in ttComponent.Handle (pcFieldName, output pcValue). 
end procedure.

procedure RepositionQuery:
    define input parameter pcClientDataName as character no-undo.
    define input parameter pcFieldName as character no-undo.
    define input parameter pcValue as character no-undo.

    find ttComponent where 
         ttComponent.Type eq 'CLIENTDATA' and
         ttComponent.Name eq pcClientDataName 
         no-lock no-error.
    if available ttComponent then
        run RepositionQuery in ttComponent.Handle (pcFieldName, pcValue). 
end procedure.

/* eof */