/*------------------------------------------------------------------------
    File        : customer_web_ui.p
    Purpose     : Presentation Layer WebSpeed UI  
    Author(s)   : pjudge
    Created     : Fri May 13 16:02:25 EDT 2011
    Notes       :
  ----------------------------------------------------------------------*/

/* Create an unnamed pool to store all the widgets created 
     by this procedure. This is a good default which assures
     that this procedure's triggers and internal procedures 
     will execute in this procedure's storage, and that proper
     cleanup will occur on deletion of the procedure. */
create widget-pool.

/* ***************************  Definitions  ************************** */
define variable mhEventHandler as handle no-undo.
define variable mcEventHandlerName as character no-undo.
define variable mlNavPanelEnabled as logical extent 4 no-undo.
define variable mcFieldName as character extent 4 no-undo.
define variable mcFieldValue as character extent 4 no-undo.
define variable mcBaseUrl as character no-undo init '/PugChallenge11/customer_eventhandler.p'.

/* ************************* Included-Libraries *********************** */
{src/web/method/cgidefs.i}

/* ************************  Main Code Block  *********************** */

/* **********************  Functions  *********************** */
function GetQueryString returns character (input pcArgs as character):
    
    define variable cQueryString as character no-undo.
    
    cQueryString = '?' /* + substitute("EventHandlerHdl=&1", mhEventHandler)
                 + '&' + substitute("EventHandlerName=&1", mhEventHandler:file-name) 
                 + '&' */ + substitute("CurrentCustNum=&1", mcFieldValue[1])
                 + '&' + left-trim(pcArgs, '&').
    
    return cQueryString.
end function.

function GetNavPanelActionHtml returns character(input pcAction as character):
    define variable cLinkHtml as character no-undo.
    define variable cActions as character no-undo init 'first,prev,next,last'.
        
    if mlNavPanelEnabled[lookup(pcAction, cActions)] then
        cLinkHtml = '<A HREF="' + mcBaseUrl + GetQueryString('&NavPanel=' + pcAction) + '">'
                  + '<IMG SRC="/img/resultset_' + pcAction + '_32x32.png" BORDER="0"></A>'.
    else
        cLinkHtml = '<IMG SRC="/img/resultset_' + pcAction + '_32x32_bw.png" BORDER="0">'.
    
    return cLinkHtml.
end function.

function GetUpdatePanelActionHtml returns character(input pcAction as character):
    define variable cLinkHtml as character no-undo.
    
    cLinkHtml = '<TD><INPUT TYPE="image" SRC="/img/' + pcAction + '_32x32.png" BORDER="0" ALT="' + pcAction + '">'.
    cLinkHtml = '<TD><IMG SRC="/img/' + pcAction + '_32x32_bw.png" BORDER="0" ALT="' + pcAction + '"/>'.
    
    return cLinkHtml.
end function.

/* ***************************  Stuff called by EventHandler  *************************** */

procedure Destroy:

end procedure.

procedure Initialise:
end procedure.

procedure BeginWaitFor:
    define input parameter pcShowMode as character no-undo.
    
    /* no-op in web world; just have the method here for the contract */
end procedure.

procedure EndWaitFor:
    /* no-op in web world */
end procedure.

procedure ShowUI:
    output-content-type ("text/html":U).

   {&OUT}         
        '<HTML>
        <HEAD>
        <TITLE>Customer Credit Limit &amp; Balance Detail</TITLE>
        </HEAD>
        <BODY>
        <CENTER>
        <FORM action="' + mcBaseUrl + GetQueryString('') + '" method="POST" name="customer_eventhandler.p">
        <TABLE>
            <TR>
                <TH>Navigation</TH>
                <TH />
                <TH>Update</TH>
            </TR>
            <TR>
                <TD>
                <TABLE>
                    <TD>' + GetNavPanelActionHtml('first') + '</TD>
                    <TD>' + GetNavPanelActionHtml('prev') + '</TD>
                    <TD>' + GetNavPanelActionHtml('next') + '</TD>
                    <TD>' + GetNavPanelActionHtml('last') + '</TD>
                </TABLE>
                </TD>
                <TD></TD>
                <TD>
                <TABLE>
                    <TD>' + GetUpdatePanelActionHtml('Add') + '</TD>
                    <TD>' + GetUpdatePanelActionHtml('Save') + '</TD>
                    <TD>' + GetUpdatePanelActionHtml('Delete') + '</TD>
                </TABLE>
                </TD>
            </TR>
        </TABLE>
        <TABLE>
            <TR>
                <TD><LABEL for="uxCustNum">Cust Num</LABEL></TD>
                <TD><INPUT type="text" name="uxCustNum" value="' + mcFieldValue[1] + '"></TD>
            </TR>
            <TR>
                <TD><LABEL for="uxCreditLimit">Credit Limit</LABEL></TD>
                <TD><INPUT type="text" name="uxCreditLimit" value="' + mcFieldValue[3] + '"></TD>
            </TR>
            <TR>
                <TD><LABEL for="uxBalance">Balance</LABEL></TD>
                <TD><INPUT type="text" name="uxBalance" value="' + mcFieldValue[4] + '"></TD>
            </TR>
            <TR>
                <TD><LABEL for="uxFullAddress">Full Address</LABEL></TD>
                <TD><INPUT type="text" name="uxFullAddress" size="75" value="' + mcFieldValue[2] + '"></TD>
            </TR>
        </TABLE>
        </FORM>
        </CENTER>
        </BODY>
        </HTML>'.
end procedure.

procedure HideUI:
    /* no-op */
end procedure.

procedure SetEventHandler:
    define input  parameter phEventHandler as handle no-undo.
    
    mhEventHandler = phEventHandler.
    mcEventHandlerName = phEventHandler:file-name.
end procedure.

procedure SetNavPanelState:
    define input  parameter plEnabled as logical extent 4 no-undo.

    mlNavPanelEnabled = plEnabled.
end procedure.

procedure ShowRecord:
    /* only the UI knows what data it wants to show */
    assign mcFieldName[1] = 'CustNum'
           mcFieldName[2] = 'FullAddress'
           mcFieldName[3] = 'CreditLimit'
           mcFieldName[4] = 'Balance'.
    
    run GetDataValue in mhEventHandler (
            'customer',
            mcFieldName,
            output mcFieldValue).
end procedure.

procedure GetScreenValue:
    define input  parameter pcFieldName as character extent no-undo.
    define output parameter pcFieldValue as character extent no-undo.
    
    define variable iLoop as integer no-undo.
    define variable iMax as integer no-undo.
    
    iMax = extent(pcFieldName).
    extent(pcFieldValue) = iMax.
    do iLoop = 1 to iMax:
        pcFieldValue[iLoop] = get-field(pcFieldName[iLoop]). 
    end.
end procedure.
