/*------------------------------------------------------------------------
    File        : customerwin.w
    Purpose     : Simple ABL window for showing customers.
    Author(s)   : pjudge
    Created     : Tue May 10 16:26:52 EDT 2011
    Notes       :
  ----------------------------------------------------------------------*/
routine-level on error undo, throw.

/* A. UI Definitions */
define button btnFirst label '|<' font 8.
define button btnPrev label '<' font 8.
define button btnNext label '>' font 8.
define button btnLast label '>|' font 8.

define button btnAdd label 'Add' font 8.
define button btnSave label 'Save' font 8.
define button btnDelete label 'Delete' font 8.

define temp-table ttCustomer no-undo before-table btCustomer
    field CustNum as integer
    field Name as character
    field CreditLimit as decimal
    field Balance as decimal
    field FullAddress as character
    index idx1 as primary unique CustNum.

define frame fCustomer
    ttCustomer.CustNum font 8
    ttCustomer.CreditLimit font 8 
    ttCustomer.Balance font 8
    ttCustomer.FullAddress format 'x(50)' font 8
    with row 5 width 200 1 col.
    
define frame fNavPanel
    btnFirst
    btnPrev
    btnNext
    btnLast
    with row 1.

define frame fUpdatePanel
    btnAdd
    btnSave
    btnDelete
    with row 1 col 50 width 60.

/* B. UI trigger definitions */
on choose of btnFirst in frame fNavPanel
do:
    run GetRecord('first').
end.

on choose of btnPrev in frame fNavPanel
do:
    run GetRecord('prev').
end.

on choose of btnNext in frame fNavPanel
do:
    run GetRecord('next').
end.

on choose of btnLast in frame fNavPanel
do:
    run GetRecord('last').
end.

on close of this-procedure
do:
    run Destroy.
    return no-apply.
end.

/** -- variables -- **/
define query qryCust for ttCustomer scrolling.
define variable hAppServer as handle no-undo.

/** -- internal procedures -- **/
procedure GetRecord:
    define input parameter pcPosition as character no-undo.
    
    case pcPosition:
        when 'first' then query qryCust:get-first().
        when 'prev' then query qryCust:get-prev().
        when 'next' then query qryCust:get-next().
        when 'last' then query qryCust:get-last().
    end case.
    
    run UpdateButtonState.    
    run ShowRecord.
end procedure.

procedure UpdateButtonState:
    /* IsFirst */
    if not query qryCust:is-open or
       query qryCust:current-result-row eq 0 then
    do with frame fNavPanel:    
        btnFirst:sensitive = false.
        btnPrev:sensitive = false.
        btnNext:sensitive = false.
        btnLast:sensitive = false.
    end.
    else       
    /*IsLast or IsFirstAndLast */
    if query qryCust:current-result-row eq query qryCust:num-results then
    do:
        /* IsFirstAndLast */
        if query qryCust:current-result-row eq 1 then
            assign btnFirst:sensitive = false
                   btnPrev:sensitive = false
                   btnNext:sensitive = false
                   btnLast:sensitive = false.
        else
            /* IsLast */
            assign btnFirst:sensitive = true
                   btnPrev:sensitive = true
                   btnNext:sensitive = false
                   btnLast:sensitive = false.
    end.
    else
    /* IsFirst */
    if query qryCust:current-result-row eq 1 then
        assign btnFirst:sensitive = false
               btnPrev:sensitive = false
               btnNext:sensitive = true
               btnLast:sensitive = true.
    else
        /* NotFirstOrLast */               
        assign btnFirst:sensitive = true
               btnPrev:sensitive = true
               btnNext:sensitive = true
               btnLast:sensitive = true.
end procedure.

procedure ShowRecord:
    displ ttCustomer.CustNum 
          ttCustomer.CreditLimit
          ttCustomer.Balance 
          ttCustomer.FullAddress
            with frame fCustomer.
end procedure.

procedure Initialise:
    current-window:width-c = 200.
    enable all with frame fNavPanel.
    
    /* get data */
    create server hAppServer.
    hAppServer:connect('-AppService asPugChallenge11 -sessionModel session-free').
    
    run data/GetCustomers.p on hAppServer (
            input 'where custnum le 100',
            output table ttCustomer).
    
    open query qryCust preselect each ttCustomer.
    
    run GetRecord('first').
    run ShowUI.
end procedure.

procedure Destroy:
    run HideUI.
    
    hAppServer:disconnect().
    quit.
end procedure.

procedure ShowUI:
    view frame fNavPanel.
    view frame fUpdatePanel.
    view frame fCustomer.
    
    current-window:title = this-procedure:file-name.
end procedure.

procedure HideUI:
    hide frame hNavPanel.
    hide frame fUpdatePanel.
    hide frame fCustomer.
    
    current-window:hidden = true.
end procedure.

/* Main block */
run Initialise.

wait-for close of this-procedure.
quit.