/*------------------------------------------------------------------------
    File        : data/GetCustomers.p
    Purpose     : Simple customer retrieval service 
    Author(s)   : pjudge
    Created     : Tue May 10 14:25:54 EDT 2011
    Notes       : * BusinessEntity, DataAccess and DataSources all rolled into one!
  ----------------------------------------------------------------------*/
  
routine-level on error undo, throw.

/** -- definitions -- **/
define temp-table ttCustomer no-undo before-table btCustomer
    field CustNum as integer
    field Name as character
    field CreditLimit as decimal
    field Balance as decimal
    field FullAddress as character 
    index idx1 as primary unique CustNum.
    
define dataset dsCustomer for ttCustomer.

/** -- parameters -- **/
define input parameter pcWhereClause as character no-undo.
define output parameter table for ttCustomer.

/** -- main -- **/
define data-source srcCustomer for Customer.

buffer ttCustomer:attach-data-source(data-source srcCustomer:handle).
data-source srcCustomer:fill-where-string = pcWhereClause.

buffer ttCustomer:set-callback-procedure ('After-Row-Fill', 'AfterRowFill_Customer').

buffer ttCustomer:fill().

buffer ttCustomer:detach-data-source().

/* manipulate the data */
procedure AfterRowFill_Customer:
    define input parameter dataset for dsCustomer.
    
    ttCustomer.FullAddress = substitute('&1, &2, &3, &5 &7, &6',
        Customer.Address,
        Customer.Address2,
        Customer.City,
        Customer.State,
        Customer.Country,
        Customer.PostalCode).
    ttCustomer.FullAddress = replace(ttCustomer.FullAddress, ' ,','').                
end procedure.
/** -- eof -- **/