/*------------------------------------------------------------------------
    File        : customer_clientdata.p
    Purpose     : 

    Syntax      :

    Description : 

    Author(s)   : pjudge
    Created     : Wed May 25 13:44:59 EDT 2011
    Notes       :
  ----------------------------------------------------------------------*/
routine-level on error undo, throw.


class CustomerClientData implements IClientData:
    define temp-table ttCustomer no-undo before-table btCustomer
        field CustNum       as integer
        field name          as character
        field CreditLimit   as decimal
        field Balance       as decimal
        field FullAddress as character
        index idx1 as primary unique CustNum.
    
    define temp-table ttTransportCustomer no-undo like ttCustomer.
    
    define private query qryCust for ttCustomer scrolling.
    define private variable hAppServer as handle no-undo.    
    
    /* A. INIT/DESTROY */
    method public void Initialise():
        create server hAppServer.
        hAppServer:connect('-AppService asPugChallenge11 -sessionModel session-free').
            
        FetchData().
    end method.
    
    method public void Destroy():
        hAppServer:disconnect().
        delete object hAppServer.
    end method.
    
    /* B. get and save DATA */
    method public void FetchData():
        run data/GetCustomers.p on hAppServer (
            input 'where custnum le 100',
            output table ttCustomer).
        
        OpenQuery().
    end method.
    
    method public void SaveData():
        /* only send changes across the wire */
        buffer ttTransportCustomer:get-changes(buffer ttCustomer:handle).
        
        run data/SaveCustomers.p on hAppServer (input table ttTransportCustomer).
        
        /* should really do error handling here! but, it compiles, so ... */
        
        OpenQuery().
    end method.
    
    /* C. BASIC DATA NAVIGATION */
    method public void OpenQuery():
        open query qryCust preselect each ttCustomer.
    end method.
    
    method public void CloseQuery():
        if query qryCust:is-open then
            close query qryCust.
    end method.
    
    method public void GetFirst():
        query qryCust:get-first().
    end method.
    
    method public void GetPrev():
        query qryCust:get-prev().
    end method.
    
    method public void GetNext():
        query qryCust:get-next().
    end method.
    
    method public void GetLast():
        query qryCust:get-last().
    end method.
    
    /* D. DATA INPUT/OUTPUT from UI */
    method public character extent GetDataValue(input pcFieldName as character extent):
        define variable cValue as character extent no-undo.
        define variable iLoop   as integer no-undo.
        define variable iMax    as integer no-undo.
        define variable hBuffer as handle  no-undo.
        
        iMax = extent (pcFieldName).
        extent(cValue) = iMax.
        do iLoop = 1 to iMax:
            cValue[iLoop] = buffer ttCustomer:buffer-field(pcFieldName[iLoop]):buffer-value.
        end.
        
        return cValue.
    end method.
    
    method public void SetValue(input pcFieldName as character extent,
                                input pcValue as character extent):
        define variable iLoop   as integer no-undo.
        define variable iMax    as integer no-undo.
        define variable hBuffer as handle  no-undo.
        
        iMax = extent (pcFieldName).
        extent(pcValue) = iMax.
        do iLoop = 1 to iMax:
            buffer ttCustomer:buffer-field(pcFieldName[iLoop]):buffer-value = pcValue[iLoop].
        end.
    end method.
    
    method public handle GetQueryHandle():
        return query qryCust:handle.
    end method.
    
    /* E. want extra enhancements - filtering, sorting, navigating around result set etc */
    method public void RepositionQuery(input pcFieldName as character,
                                       input pcValue as character):
        define buffer lbRepos for ttCustomer.
        
        case pcFieldName:
            when 'CustNum' then 
                find lbRepos where lbRepos.CustNum eq int(pcValue) no-error.
        end.
        
        if available lbRepos then
        do:
            query qryCust:reposition-to-rowid(rowid(lbRepos)).
            query qryCust:get-next().
        end.
    end method.
    
    method public void FilterQuery(input pcFilter as character):
        define variable hQuery as handle no-undo.
        
        hQuery = query qryCust:handle.
        
        hQuery:query-prepare(substitute(' preselect each ttCustomer &1', pcFilter)).
        
        OpenQuery().
    end method.
    
    method public void SortQuery(input pcSort as character):
        define variable hQuery as handle no-undo.
        
        hQuery = query qryCust:handle.
        
        hQuery:query-prepare(substitute(' preselect each ttCustomer by &1', pcSort)).
        
        OpenQuery().
    end method.
    
    method public character GetCurrentPosition():
        define variable cCurrentPosition as character no-undo.
    
        /* IsFirst */
        if not query qryCust:is-open or
            query qryCust:current-result-row eq 0 then
            cCurrentPosition = 'None'.
        else       
            /*IsLast or IsFirstAndLast */
            if query qryCust:current-result-row eq query qryCust:num-results then
            do:
                /* IsFirstAndLast */
                if query qryCust:current-result-row eq 1 then
                    cCurrentPosition = 'IsFirstAndLast'. 
                else
                    cCurrentPosition = 'IsLast'.
            end.
            else
                /* IsFirst */
                if query qryCust:current-result-row eq 1 then
                    cCurrentPosition = 'IsFirst'. 
                else
                    cCurrentPosition = 'NotFirstOrLast'.
        return cCurrentPosition.
    end method.
    
end class.
    